@@ -161,19 +161,37 @@ const char *GWEN_I18N_GetCurrentLocale() {
 
 
 
-const char *GWEN_I18N_Translate(const char *textdomain, const char *text) {
+const char *GWEN_I18N_Translate(const char *txtdom, const char *text) {
 #ifdef HAVE_I18N
-  return dgettext(textdomain, text);
+  const char *p;
+
+  p=strchr(text, '|');
+  if (p) {
+    const char *s;
+
+    s=dgettext(txtdom, text);
+    if (strcmp(s, text)==0)
+      return ++p;
+    else
+      return s;
+  }
+  else
+    return dgettext(txtdom, text);
 #else
+  const char *p;
+
+  p=strchr(text, '|');
+  if (p)
+    return ++p;
   return text;
 #endif
 }
 
 
 
-int GWEN_I18N_BindTextDomain_Dir(const char *textdomain, const char *folder) {
+int GWEN_I18N_BindTextDomain_Dir(const char *txtdom, const char *folder) {
 #ifdef HAVE_I18N
-  if (NULL==bindtextdomain(textdomain, folder)) {
+  if (NULL==bindtextdomain(txtdom, folder)) {
     DBG_INFO(GWEN_LOGDOMAIN, "bindtextdomain(): %s", strerror(errno));
     return GWEN_ERROR_GENERIC;
   }
@@ -185,9 +203,9 @@ int GWEN_I18N_BindTextDomain_Dir(const char *textdomain, const char *folder) {
 
 
 
-int GWEN_I18N_BindTextDomain_Codeset(const char *textdomain, const char *cs) {
+int GWEN_I18N_BindTextDomain_Codeset(const char *txtdom, const char *cs) {
 #ifdef HAVE_I18N
-  if (NULL==bind_textdomain_codeset(textdomain, cs)) {
+  if (NULL==bind_textdomain_codeset(txtdom, cs)) {
     DBG_INFO(GWEN_LOGDOMAIN, "bind_textdomain_codeset(): %s", strerror(errno));
     return GWEN_ERROR_GENERIC;
   }
