@@ -453,7 +453,7 @@ int GWEN_Crypt_KeyRsa__WriteMpi(GWEN_DB_NODE *db,
 
 
 int GWEN_Crypt_KeyRsa__DataFromDb(GWEN_DB_NODE *db, gcry_ac_data_t *pData,
-				  int pub, unsigned int nbits) {
+				  int pub, GWEN_UNUSED unsigned int nbits) {
   gcry_ac_data_t ds;
   gcry_error_t err;
   int rv;
@@ -488,17 +488,20 @@ int GWEN_Crypt_KeyRsa__DataFromDb(GWEN_DB_NODE *db, gcry_ac_data_t *pData,
     }
 
     /* read p */
-    rv=GWEN_Crypt_KeyRsa__ReadMpi(db, "p", ds, "p");
-    if (rv) {
-      gcry_ac_data_destroy(ds);
-      return rv;
-    }
+    if (GWEN_DB_VariableExists(db, "p") &&
+	GWEN_DB_VariableExists(db, "q")) {
+      rv=GWEN_Crypt_KeyRsa__ReadMpi(db, "p", ds, "p");
+      if (rv) {
+	gcry_ac_data_destroy(ds);
+	return rv;
+      }
 
-    /* read q */
-    rv=GWEN_Crypt_KeyRsa__ReadMpi(db, "q", ds, "q");
-    if (rv) {
-      gcry_ac_data_destroy(ds);
-      return rv;
+      /* read q */
+      rv=GWEN_Crypt_KeyRsa__ReadMpi(db, "q", ds, "q");
+      if (rv) {
+	gcry_ac_data_destroy(ds);
+	return rv;
+      }
     }
   }
 
@@ -509,7 +512,7 @@ int GWEN_Crypt_KeyRsa__DataFromDb(GWEN_DB_NODE *db, gcry_ac_data_t *pData,
 
 
 GWENHYWFAR_CB
-void GWEN_Crypt_KeyRsa_freeData(void *bp, void *p) {
+void GWEN_Crypt_KeyRsa_freeData(GWEN_UNUSED void *bp, void *p) {
   GWEN_CRYPT_KEY_RSA *xk;
 
   xk=(GWEN_CRYPT_KEY_RSA*) p;
@@ -658,23 +661,29 @@ int GWEN_Crypt_KeyRsa_toDb(const GWEN_CRYPT_KEY *k, GWEN_DB_NODE *db, int pub) {
     return rv;
   }
   if (!pub) {
+    gcry_mpi_t mpi;
+
     /* store d */
     rv=GWEN_Crypt_KeyRsa__WriteMpi(dbR, "d", ds, "d");
     if (rv) {
       DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
       return rv;
     }
-    /* store p */
-    rv=GWEN_Crypt_KeyRsa__WriteMpi(dbR, "p", ds, "p");
-    if (rv) {
-      DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-      return rv;
-    }
-    /* store q */
-    rv=GWEN_Crypt_KeyRsa__WriteMpi(dbR, "q", ds, "q");
-    if (rv) {
-      DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-      return rv;
+
+    if (gcry_ac_data_get_name(ds, 0, "p", &mpi)==0 &&
+	gcry_ac_data_get_name(ds, 0, "q", &mpi)==0) {
+      /* store p */
+      rv=GWEN_Crypt_KeyRsa__WriteMpi(dbR, "p", ds, "p");
+      if (rv) {
+	DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+	return rv;
+      }
+      /* store q */
+      rv=GWEN_Crypt_KeyRsa__WriteMpi(dbR, "q", ds, "q");
+      if (rv) {
+	DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+	return rv;
+      }
     }
   }
 
@@ -746,16 +755,20 @@ int GWEN_Crypt_KeyRsa__sKeyToDataPrivKey(gcry_ac_data_t data, gcry_sexp_t sx) {
 
 
 
-int GWEN_Crypt_KeyRsa_GeneratePair(unsigned int nbytes, int use65537e,
-				   GWEN_CRYPT_KEY **pPubKey,
-				   GWEN_CRYPT_KEY **pSecretKey) {
+int GWEN_Crypt_KeyRsa_GeneratePair2(unsigned int nbits, int use65537e,
+				    GWEN_CRYPT_KEY **pPubKey,
+				    GWEN_CRYPT_KEY **pSecretKey) {
   gcry_sexp_t keyparm, key;
   int rc;
   char buffer[256];
   char numbuf[32];
   gcry_sexp_t skey, pkey;
+  int nbytes;
 
-  snprintf(numbuf, sizeof(numbuf)-1, "%d", nbytes*8);
+  nbytes=nbits/8;
+  if (nbits%8)
+    nbytes++;
+  snprintf(numbuf, sizeof(numbuf)-1, "%d", nbits);
   if (use65537e) {
     snprintf(buffer, sizeof(buffer)-1,
 	     "(genkey\n"
@@ -764,7 +777,7 @@ int GWEN_Crypt_KeyRsa_GeneratePair(unsigned int nbytes, int use65537e,
 	     "  (rsa-use-e 5:65537)\n"
 	     " ))",
 	     strlen(numbuf),
-	     nbytes*8);
+	     nbits);
   }
   else
     snprintf(buffer, sizeof(buffer)-1,
@@ -774,7 +787,10 @@ int GWEN_Crypt_KeyRsa_GeneratePair(unsigned int nbytes, int use65537e,
 	     "  (rsa-use-e 1:0)\n"
 	     " ))",
 	     strlen(numbuf),
-	     nbytes*8);
+	     nbits);
+  buffer[sizeof(buffer)-1]=0;
+
+  /*DBG_ERROR(0, "Genkey string: [%s]", buffer);*/
 
   rc=gcry_sexp_new(&keyparm, buffer, 0, 1);
   if (rc) {
@@ -923,6 +939,13 @@ int GWEN_Crypt_KeyRsa_GeneratePair(unsigned int nbytes, int use65537e,
 
 
 
+int GWEN_Crypt_KeyRsa_GeneratePair(unsigned int nbytes, int use65537e,
+				   GWEN_CRYPT_KEY **pPubKey,
+				   GWEN_CRYPT_KEY **pSecretKey) {
+  return GWEN_Crypt_KeyRsa_GeneratePair2(nbytes*8, use65537e, pPubKey, pSecretKey);
+}
+
+
 int GWEN_Crypt_KeyRsa__GetNamedElement(const GWEN_CRYPT_KEY *k,
                                        const char *name,
 				       uint8_t *buffer,
@@ -1045,6 +1068,64 @@ GWEN_CRYPT_KEY *GWEN_Crypt_KeyRsa_fromModExp(unsigned int nbytes,
 
 
 
+GWEN_CRYPT_KEY *GWEN_Crypt_KeyRsa_fromModPrivExp(unsigned int nbytes,
+						 const uint8_t *pModulus,
+						 uint32_t lModulus,
+						 const uint8_t *pExponent,
+						 uint32_t lExponent,
+						 const uint8_t *pPrivExponent,
+						 uint32_t lPrivExponent) {
+  GWEN_DB_NODE *dbKey;
+  GWEN_DB_NODE *dbR;
+  GWEN_CRYPT_KEY *key;
+
+  assert(nbytes);
+  assert(pModulus);
+  assert(lModulus);
+  assert(pExponent);
+  assert(lExponent);
+  assert(pPrivExponent);
+  assert(lPrivExponent);
+
+  dbKey=GWEN_DB_Group_new("key");
+  dbR=GWEN_DB_GetGroup(dbKey, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "rsa");
+
+  /* basic key stuff */
+  GWEN_DB_SetCharValue(dbKey, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		       "cryptAlgoId",
+		       GWEN_Crypt_CryptAlgoId_toString(GWEN_Crypt_CryptAlgoId_Rsa));
+  GWEN_DB_SetIntValue(dbKey, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		      "keySize", nbytes);
+
+  /* RSA stuff */
+  GWEN_DB_SetIntValue(dbR, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		      "isPublic", 0);
+  GWEN_DB_SetBinValue(dbR, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		      "n",
+                      pModulus, lModulus);
+  GWEN_DB_SetBinValue(dbR, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		      "e",
+                      pExponent, lExponent);
+  GWEN_DB_SetBinValue(dbR, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		      "d",
+		      pPrivExponent, lPrivExponent);
+
+  /* create key from DB */
+  key=GWEN_Crypt_KeyRsa_fromDb(dbKey);
+  if (key==NULL) {
+    DBG_INFO(GWEN_LOGDOMAIN,
+	     "Internal error: Bad RSA key group");
+    GWEN_DB_Dump(dbKey, stderr, 2);
+    GWEN_DB_Group_free(dbKey);
+    return NULL;
+  }
+
+  GWEN_DB_Group_free(dbKey);
+  return key;
+}
+
+
+
 GWEN_CRYPT_KEY *GWEN_Crypt_KeyRsa_dup(const GWEN_CRYPT_KEY *k) {
   GWEN_CRYPT_KEY_RSA *xk;
   GWEN_DB_NODE *dbKey;
