@@ -44,12 +44,19 @@ int GWEN_Crypt_TokenFile__OpenFile(GWEN_CRYPT_TOKEN *ct, int wr, uint32_t gid){
   int fd;
   GWEN_CRYPT_TOKEN_FILE *lct;
   GWEN_FSLOCK_RESULT lres;
+  const char *fname;
 
   assert(ct);
   lct=GWEN_INHERIT_GETDATA(GWEN_CRYPT_TOKEN, GWEN_CRYPT_TOKEN_FILE, ct);
   assert(lct);
 
-  lct->lock=GWEN_FSLock_new(GWEN_Crypt_Token_GetTokenName(ct),
+  fname = GWEN_Crypt_Token_GetTokenName(ct);
+  if (!fname) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "No name of the crypt token set - maybe you need to set the key file as token name? Cannot lock token.");
+    return GWEN_ERROR_IO;
+  }
+
+  lct->lock=GWEN_FSLock_new(fname,
 			    GWEN_FSLock_TypeFile);
   lres=GWEN_FSLock_Lock(lct->lock, 10000, gid);
   if (lres!=GWEN_FSLock_ResultOk) {
@@ -1917,7 +1924,6 @@ GWEN_Crypt_TokenFile__GenerateKey(GWEN_CRYPT_TOKEN *ct,
       GWEN_Crypt_KeyRsa_AddFlags(secKey, GWEN_CRYPT_KEYRSA_FLAGS_DIRECTSIGN);
     }
     GWEN_CTF_Context_SetLocalSignKey(ctx, secKey);
-    GWEN_CTF_Context_SetLocalSignKeyInfo(ctx, ki);
     GWEN_Crypt_Token_KeyInfo_AddFlags(ki,
 				      GWEN_CRYPT_TOKEN_KEYFLAGS_HASMODULUS |
 				      GWEN_CRYPT_TOKEN_KEYFLAGS_HASEXPONENT |
@@ -1927,10 +1933,11 @@ GWEN_Crypt_TokenFile__GenerateKey(GWEN_CRYPT_TOKEN *ct,
 				      GWEN_CRYPT_TOKEN_KEYFLAGS_HASACTIONFLAGS |
 				      GWEN_CRYPT_TOKEN_KEYFLAGS_CANVERIFY |
 				      GWEN_CRYPT_TOKEN_KEYFLAGS_CANSIGN);
+    GWEN_Crypt_Token_KeyInfo_SetSignCounter(ki, 1);
+    GWEN_CTF_Context_SetLocalSignKeyInfo(ctx, ki);
   }
   else if (keyNum==2) {
     GWEN_CTF_Context_SetLocalCryptKey(ctx, secKey);
-    GWEN_CTF_Context_SetLocalCryptKeyInfo(ctx, ki);
     GWEN_Crypt_Token_KeyInfo_AddFlags(ki,
 				      GWEN_CRYPT_TOKEN_KEYFLAGS_HASMODULUS |
 				      GWEN_CRYPT_TOKEN_KEYFLAGS_HASEXPONENT |
@@ -1940,6 +1947,7 @@ GWEN_Crypt_TokenFile__GenerateKey(GWEN_CRYPT_TOKEN *ct,
 				      GWEN_CRYPT_TOKEN_KEYFLAGS_HASACTIONFLAGS |
 				      GWEN_CRYPT_TOKEN_KEYFLAGS_CANENCIPHER |
 				      GWEN_CRYPT_TOKEN_KEYFLAGS_CANDECIPHER);
+    GWEN_CTF_Context_SetLocalCryptKeyInfo(ctx, ki);
   }
   else {
     if (GWEN_Crypt_Token_GetModes(ct) & GWEN_CRYPT_TOKEN_MODE_DIRECT_SIGN){
@@ -1947,7 +1955,6 @@ GWEN_Crypt_TokenFile__GenerateKey(GWEN_CRYPT_TOKEN *ct,
       GWEN_Crypt_KeyRsa_AddFlags(secKey, GWEN_CRYPT_KEYRSA_FLAGS_DIRECTSIGN);
     }
     GWEN_CTF_Context_SetLocalAuthKey(ctx, secKey);
-    GWEN_CTF_Context_SetLocalAuthKeyInfo(ctx, ki);
     GWEN_Crypt_Token_KeyInfo_AddFlags(ki,
 				      GWEN_CRYPT_TOKEN_KEYFLAGS_HASMODULUS |
 				      GWEN_CRYPT_TOKEN_KEYFLAGS_HASEXPONENT |
@@ -1957,6 +1964,8 @@ GWEN_Crypt_TokenFile__GenerateKey(GWEN_CRYPT_TOKEN *ct,
 				      GWEN_CRYPT_TOKEN_KEYFLAGS_HASACTIONFLAGS |
 				      GWEN_CRYPT_TOKEN_KEYFLAGS_CANVERIFY |
 				      GWEN_CRYPT_TOKEN_KEYFLAGS_CANSIGN);
+    GWEN_Crypt_Token_KeyInfo_SetSignCounter(ki, 1);
+    GWEN_CTF_Context_SetLocalAuthKeyInfo(ctx, ki);
   }
 
   /* the public key is not used */
