@@ -28,7 +28,9 @@
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
-#else
+#endif
+
+#ifndef ICONV_CONST
 # define ICONV_CONST
 #endif
 
@@ -863,9 +865,9 @@ int GWEN_Gui_CGui_CheckCert(GWEN_GUI *gui,
       return 0;
     }
     else {
-      DBG_NOTICE(GWEN_LOGDOMAIN,
-		 "Automatically rejecting certificate [%s] (noninteractive)",
-		 hash);
+      DBG_ERROR(GWEN_LOGDOMAIN,
+		"Automatically rejecting certificate [%s] (noninteractive)",
+		hash);
       GWEN_Buffer_free(hbuf);
       return GWEN_ERROR_USER_ABORTED;
     }
@@ -972,13 +974,17 @@ int GWEN_Gui_CGui_GetPassword(GWEN_GUI *gui,
 	  GWEN_Buffer_free(buf);
 	  return 0;
 	}
+	else {
+	  DBG_ERROR(GWEN_LOGDOMAIN, "Stored password [%s] is not within size limits (%d), rejecting.",
+		    GWEN_Buffer_GetStart(buf), i);
+	}
       }
     }
 
     if (GWEN_Gui_GetFlags(gui) & GWEN_GUI_FLAGS_NONINTERACTIVE) {
       DBG_ERROR(GWEN_LOGDOMAIN,
 		"Password for [%s] missing in noninteractive mode, "
-		"aborting", token);
+		"aborting", GWEN_Buffer_GetStart(buf));
       GWEN_Buffer_free(buf);
       return GWEN_ERROR_USER_ABORTED;
     }
