@@ -18,7 +18,8 @@
 extern "C" {
 #endif
 
-/** @defgroup MOD_CGUI GUI Implementation for the Console
+/** @defgroup MOD_GUI_CGUI GUI Implementation for the Console
+ * @ingroup MOD_GUI
  *
  * This is an implementation of GWEN_GUI for the console. It supports
  * interactive and non-interactive mode, provides a cache for passwords/pins
@@ -70,10 +71,10 @@ void GWEN_Gui_CGui_SetCharSet(GWEN_GUI *gui, const char *s);
  * These settings together allow for a non-interactive use.
  */
 /*@{*/
-GWENHYWFAR_API 
+GWENHYWFAR_API DEPRECATED
 int GWEN_Gui_CGui_GetIsNonInteractive(const GWEN_GUI *gui);
 
-GWENHYWFAR_API 
+GWENHYWFAR_API DEPRECATED
 void GWEN_Gui_CGui_SetIsNonInteractive(GWEN_GUI *gui, int i);
 /*@}*/
 
@@ -133,6 +134,23 @@ void GWEN_Gui_CGui_SetCertDb(GWEN_GUI *gui, GWEN_DB_NODE *dbCerts);
  */
 GWENHYWFAR_API 
 GWEN_DB_NODE *GWEN_Gui_CGui_GetCertDb(const GWEN_GUI *gui);
+
+/**
+ * In non-interactive mode only known certificates are accepted.
+ * If the parameter i unequals zero new certs are also accepted if they
+ * are valid (which means signed by a known and trusted authority, not expired
+ * etc).
+ * Invalid certificates are always rejected in non-interactive mode.
+ */
+GWENHYWFAR_API DEPRECATED
+void GWEN_Gui_CGui_SetAcceptAllValidCerts(GWEN_GUI *gui, int i);
+
+/**
+ * See @ref GWEN_Gui_CGui_SetAcceptAllValidCerts
+ */
+GWENHYWFAR_API DEPRECATED
+int GWEN_Gui_CGui_GetAcceptAllValidCerts(const GWEN_GUI *gui);
+
 /*@}*/
 
 
