@@ -281,6 +281,37 @@ GWEN_Gui_SetKeyDataFromTextOpenSslFn(GWEN_GUI *gui,
 
 
 
+uint32_t GWEN_Gui_GetFlags(const GWEN_GUI *gui) {
+  assert(gui);
+  return gui->flags;
+}
+
+
+
+void GWEN_Gui_SetFlags(GWEN_GUI *gui, uint32_t fl) {
+  assert(gui);
+  gui->flags=fl;
+}
+
+
+
+void GWEN_Gui_AddFlags(GWEN_GUI *gui, uint32_t fl) {
+  assert(gui);
+  gui->flags|=fl;
+}
+
+
+
+void GWEN_Gui_SubFlags(GWEN_GUI *gui, uint32_t fl) {
+  assert(gui);
+  gui->flags&=~fl;
+}
+
+
+
+
+
+
 
 
 
@@ -427,7 +458,8 @@ int GWEN_Gui_GetPassword(uint32_t flags,
 					   maxLen,
 					   guiid);
     else
-      return gwenhywfar_gui->inputBoxFn(gwenhywfar_gui,
+      if (gwenhywfar_gui->inputBoxFn)
+	return gwenhywfar_gui->inputBoxFn(gwenhywfar_gui,
 					flags,
 					title,
 					text,
@@ -455,8 +487,19 @@ int GWEN_Gui_SetPasswordStatus(const char *token,
 
 int GWEN_Gui_LogHook(const char *logDomain,
 		     GWEN_LOGGER_LEVEL priority, const char *s) {
-  if (gwenhywfar_gui && gwenhywfar_gui->logHookFn)
-    return gwenhywfar_gui->logHookFn(gwenhywfar_gui, logDomain, priority, s);
+  if (gwenhywfar_gui && gwenhywfar_gui->logHookFn) {
+    if (priority>=GWEN_LoggerLevel_Debug &&
+	logDomain &&
+	strcasecmp(logDomain, "gwenhywfar")==0)
+      /* don't send possibly sensitive data to the log function because
+       * some application tend to store the messages indiscriminately.
+       * In some cases sensitive information can be send to this function
+       * which we don't want the application to store */
+      return 0;
+    else
+      /* otherwise the log message seems to be uncritical, convey it */
+      return gwenhywfar_gui->logHookFn(gwenhywfar_gui, logDomain, priority, s);
+  }
   else
     /* handle as usual */
     return 0;
@@ -544,9 +587,9 @@ int GWEN_Gui_CheckCert(const GWEN_SSLCERTDESCR *cd, GWEN_IO_LAYER *io, uint32_t
 
 
 
-int GWEN_Gui_CheckCertBuiltIn(GWEN_GUI *gui,
+int GWEN_Gui_CheckCertBuiltIn(GWEN_UNUSED GWEN_GUI *gui,
 			      const GWEN_SSLCERTDESCR *cd,
-			      GWEN_IO_LAYER *io, uint32_t guiid) {
+			      GWEN_UNUSED GWEN_IO_LAYER *io, uint32_t guiid) {
   int rv;
   int isError;
   const char *hash;
