@@ -310,6 +310,7 @@ int GWEN_Io_LayerTls_Prepare(GWEN_IO_LAYER *io) {
   if (lflags & GWEN_IO_LAYER_TLS_FLAGS_FORCE_SSL_V3) {
     const int proto_prio[2] = { GNUTLS_SSL3, 0 };
 
+    DBG_INFO(GWEN_LOGDOMAIN, "Forcing SSL v3");
     rv=gnutls_protocol_set_priority(xio->session, proto_prio);
     if (rv) {
       DBG_ERROR(GWEN_LOGDOMAIN, "gnutls_protocol_set_priority: %d (%s)", rv, gnutls_strerror(rv));
@@ -840,7 +841,7 @@ ssize_t GWEN_Io_LayerTls_Push(gnutls_transport_ptr_t p, const void *buf, size_t
 
   rv=GWEN_Io_LayerCodec_CheckWriteOut(io);
   if (rv) {
-    if (rv==GWEN_ERROR_TRY_AGAIN) {
+    if (rv==GWEN_ERROR_TRY_AGAIN || rv==GWEN_ERROR_IN_PROGRESS) {
       DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
 #ifdef HAVE_GNUTLS_TRANSPORT_SET_ERRNO
       gnutls_transport_set_errno(xio->session, EAGAIN);
@@ -1080,17 +1081,11 @@ int GWEN_Io_LayerTls_DelRequest(GWEN_IO_LAYER *io, GWEN_IO_REQUEST *r) {
 
   case GWEN_Io_Request_TypeDisconnect:
     if (xio->disconnectRequest==r) {
-      DBG_DEBUG(GWEN_LOGDOMAIN, "Aborted connect request");
+      DBG_DEBUG(GWEN_LOGDOMAIN, "Aborted disconnect request");
 
       GWEN_Io_Layer_SetStatus(io, GWEN_Io_Layer_StatusDisconnected);
-      if (xio->connectRequest) {
-	GWEN_IO_REQUEST *r;
-
-	r=xio->connectRequest;
-	xio->connectRequest=NULL;
-	GWEN_Io_Request_Finished(r, GWEN_Io_Request_StatusFinished, GWEN_ERROR_ABORTED);
-	GWEN_Io_Request_free(r);
-      }
+      GWEN_Io_Request_Finished(r, GWEN_Io_Request_StatusFinished, GWEN_ERROR_ABORTED);
+      GWEN_Io_Request_free(r);
     }
     else {
       /* not my request */
@@ -1121,6 +1116,9 @@ int GWEN_Io_LayerTls_HasWaitingRequests(GWEN_IO_LAYER *io) {
   xio=GWEN_INHERIT_GETDATA(GWEN_IO_LAYER, GWEN_IO_LAYER_TLS, io);
   assert(xio);
 
+  if (xio->waitingForGui)
+    return 0;
+
   if (xio->connectRequest || xio->disconnectRequest)
     return 1;
   if (xio->hasWaitingRequestsFn)
@@ -1139,6 +1137,10 @@ GWEN_IO_LAYER_WORKRESULT GWEN_Io_LayerTls_WorkOnRequests(GWEN_IO_LAYER *io) {
   xio=GWEN_INHERIT_GETDATA(GWEN_IO_LAYER, GWEN_IO_LAYER_TLS, io);
   assert(xio);
 
+  if (xio->waitingForGui)
+    /* this worker is waiting for GUI, don't actually work here */
+    return GWEN_Io_Layer_WorkResultBlocking;
+
   /* work on connect request */
   if (xio->connectRequest) {
     GWEN_IO_REQUEST *r;
@@ -1152,6 +1154,31 @@ GWEN_IO_LAYER_WORKRESULT GWEN_Io_LayerTls_WorkOnRequests(GWEN_IO_LAYER *io) {
 	  rv!=GNUTLS_E_INTERRUPTED) {
 	DBG_ERROR(GWEN_LOGDOMAIN, "gnutls_handshake: %d (%s) [%s]",
 		  rv, gnutls_strerror(rv), gnutls_error_is_fatal(rv)?"fatal":"non-fatal");
+	if (rv==GNUTLS_E_UNEXPECTED_PACKET_LENGTH) {
+	  GWEN_Gui_ProgressLog(GWEN_Io_Request_GetGuiId(r),
+			       GWEN_LoggerLevel_Error,
+			       I18N("A TLS handshake error occurred. "
+				    "If you are using AqBanking you should "
+				    "consider enabling the option "
+				    "\"force SSLv3\" in the user settings "
+				    "dialog."));
+	}
+	else {
+	  GWEN_BUFFER *tbuf;
+          char numbuf[32];
+
+	  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
+	  GWEN_Buffer_AppendString(tbuf, I18N("TLS Handshake Error:"));
+	  snprintf(numbuf, sizeof(numbuf)-1, " %d ", rv);
+	  GWEN_Buffer_AppendString(tbuf, numbuf);
+	  GWEN_Buffer_AppendString(tbuf, "(");
+	  GWEN_Buffer_AppendString(tbuf, gnutls_strerror(rv));
+	  GWEN_Buffer_AppendString(tbuf, ")");
+	  GWEN_Gui_ProgressLog(GWEN_Io_Request_GetGuiId(r),
+			       GWEN_LoggerLevel_Error,
+			       GWEN_Buffer_GetStart(tbuf));
+          GWEN_Buffer_free(tbuf);
+	}
 	GWEN_Io_Layer_SetStatus(io, GWEN_Io_Layer_StatusDisconnected);
 	xio->connectRequest=NULL;
 	GWEN_Io_Request_Finished(r, GWEN_Io_Request_StatusFinished, GWEN_ERROR_SSL);
@@ -1191,8 +1218,10 @@ GWEN_IO_LAYER_WORKRESULT GWEN_Io_LayerTls_WorkOnRequests(GWEN_IO_LAYER *io) {
 	}
       }
       else {
-	/* present cert to the user */
+	  /* present cert to the user */
+	xio->waitingForGui=1;
 	rv=GWEN_Gui_CheckCert(xio->peerCertDescr, io, GWEN_Io_Request_GetGuiId(r));
+	xio->waitingForGui=0;
 	if (rv) {
 	  DBG_INFO(GWEN_LOGDOMAIN, "Peer cert not accepted (%d), aborting", rv);
 	  GWEN_Io_Layer_SetStatus(io, GWEN_Io_Layer_StatusDisconnected);
