@@ -469,6 +469,31 @@ double GWEN_Time_DiffSeconds(const GWEN_TIME *t1, const GWEN_TIME *t0){
 
 
 
+int GWEN_Time_Compare(const GWEN_TIME *t1, const GWEN_TIME *t0){
+  if (t1 && t0) {
+    if (t1->secs<t0->secs)
+      return -1;
+    else if (t1->secs>t0->secs)
+      return 1;
+    else {
+      if (t1->msecs<t0->msecs)
+	return -1;
+      else if (t1->msecs>t0->msecs)
+	return 1;
+      else
+	return 0;
+    }
+  }
+  else if (t1)
+    return 1;
+  else if (t0)
+    return -1;
+
+  return 0;
+}
+
+
+
 double GWEN_Time_Milliseconds(const GWEN_TIME *t){
   assert(t);
   return (double)((t->secs*1000)+(t->msecs));
@@ -571,6 +596,7 @@ int GWEN_Time_GetBrokenDownUtcDate(const GWEN_TIME *t,
 
 
 
+/* TODO: compiler says "function returns an aggregate" */
 struct tm GWEN_Time_toTm(const GWEN_TIME *t) {
   struct tm *tb;
   time_t tt;
@@ -626,8 +652,8 @@ GWEN_TIME_TMPLCHAR *GWEN_Time__findTmplChar(GWEN_TIME_TMPLCHAR_LIST *ll,
 
 
 
-void GWEN_Time__sampleTmplChars(const GWEN_TIME *t, const char *tmpl,
-                                GWEN_BUFFER *buf,
+void GWEN_Time__sampleTmplChars(GWEN_UNUSED const GWEN_TIME *t, const char *tmpl,
+                                GWEN_UNUSED GWEN_BUFFER *buf,
                                 GWEN_TIME_TMPLCHAR_LIST *ll) {
   const char *s;
 
