@@ -207,7 +207,6 @@ int GWEN_XMLNode_WriteToStream(const GWEN_XMLNODE *n,
     nn=nheader;
     while(nn) {
       const GWEN_XMLNODE *next;
-      int rv;
 
       rv=GWEN_XMLNode__WriteToStream(nn, fb, GWEN_XmlCtx_GetFlags(ctx), 0);
       if (rv<0) {
@@ -369,7 +368,7 @@ int GWEN_XMLNode_toBuffer(const GWEN_XMLNODE *n, GWEN_BUFFER *buf, uint32_t flag
 
 int GWEN_XML__ReadData(GWEN_XML_CONTEXT *ctx,
 		       GWEN_FAST_BUFFER *fb,
-		       uint32_t flags){
+		       GWEN_UNUSED uint32_t flags){
   int chr;
   unsigned char uc;
   GWEN_BUFFER *dbuf;
@@ -412,7 +411,7 @@ int GWEN_XML__ReadData(GWEN_XML_CONTEXT *ctx,
 
 int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
 		      GWEN_FAST_BUFFER *fb,
-		      uint32_t flags){
+		      GWEN_UNUSED uint32_t flags){
   int chr;
   unsigned char uc=0;
   GWEN_BUFFER *dbuf;
@@ -745,6 +744,8 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
   }
 
   if (uc=='?' || uc=='!' || uc=='/') {
+    unsigned char ucsave=uc;
+
     GWEN_FASTBUFFER_PEEKBYTE(fb, chr);
     if (chr<0) {
       GWEN_Buffer_free(dbuf);
@@ -763,6 +764,12 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
       /* tag finished */
       return 0;
     }
+    else {
+      DBG_ERROR(GWEN_LOGDOMAIN,
+		"Got an unexpected character here (after %02x[%c]): %02x[%c], "
+		"maybe the text contains unescaped XML characters?",
+		ucsave, ucsave, uc, uc);
+    }
   }
   else if (uc=='>') {
     rv=GWEN_XmlCtx_EndTag(ctx, 0);
